/*------------------------------------------------------------ -*- C -*-
 *  2 Kernel Monte a.k.a. Linux loading Linux on x86
 *  monte.c:  User level utility to use kernel monte.
 *
 *  Copyright (C) 2000 Erik Arjan Hendriks <hendriks@scyld.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: monte.c,v 1.4 2000/03/14 18:39:06 hendriks Exp $
 *--------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <syscall.h>

#define MONTE_MAGIC_1 0xdeaddead
#define MONTE_MAGIC_2 0xdeaddead
struct monte_param_t {
    char *command_line;
    int   kernel_fd;
    int   initrd_fd;

    /* XXX We should have an interface for copying a ramdisk or an
     * initrd from user space. */
};

void usage(char *arg0) {
    fprintf(stderr,
"Usage: %s [-h] [-v] [-i initrd] imagefile [command line ....]\n"
"       This program loads a Linux kernel image and optionally an initial\n"
"       ramdisk image into memory and restarts the machine using the\n"
"       new kernel.\n\n"
"       -h          Display this message and exit.\n"
"       -v          Display the program version number and exit.\n"
"       -i initrd   Load the initial ram disk image stored in initrd.\n\n"
"       imagefile is the kernel image file to be loaded.  All remaining\n"
"       options on the command line will be concatenated and passed to\n"
"       the kernel as the kernel command line.\n"
	    , arg0);
}

int main(int argc, char *argv[]) {
    int i, len, c;
    char *initrd_name=0;
    struct monte_param_t param;

    param.kernel_fd = -1;
    param.initrd_fd = -1;

    while((c = getopt(argc, argv, "i:vh"))!=EOF) {
	switch(c) {
	case 'h':
	    usage(argv[0]);
	    exit(0);
	case 'v':
	    printf("%s: version %s\n", argv[0], PACKAGE_VERSION);
	    exit(0);
	case 'i':
	    if (initrd_name) {
		fprintf(stderr, "%s: Only one initrd image allowed.\n", argv[0]);
		exit(1);
	    }
	    initrd_name = optarg;
	    break;
	default:
	    usage(argv[0]);
	    exit(1);
	}
    }

    if (argc - optind == 0) {
	usage(argv[0]);
	exit(1);
    }

    /* Open the kernel image file */
    if (strcmp(argv[optind], "-") == 0) {
	param.kernel_fd = STDIN_FILENO;
    } else {
	param.kernel_fd = open(argv[optind], O_RDONLY);
	if (param.kernel_fd == -1) {
	    perror(argv[optind]);
	    exit(1);
	}
    }

    /* Open the initrd image file */
    if (initrd_name) {
	if (strcmp(initrd_name, "-") == 0) {
	    param.initrd_fd = STDIN_FILENO;
	} else {
	    param.initrd_fd = open(initrd_name, O_RDONLY);
	    if (param.initrd_fd == -1) {
		perror(initrd_name);
		exit(1);
	    }
	}
    }

    /* The rest of the arguments get assembled into the the kernel
     * command line */
    len = 1;
    for (i=optind+1; i < argc; i++)
	len += strlen(argv[i])+1;
    param.command_line = malloc (len);
    if (!param.command_line) {
	fprintf(stderr, "Out of memory.\n");
	exit(1);
    }
    param.command_line[0] = 0;
    for (i=optind+1; i < argc; i++) {
	strcat(param.command_line, argv[i]);
	if (i+1 < argc) strcat(param.command_line, " ");
    }
    
    syscall(__NR_reboot, MONTE_MAGIC_1, MONTE_MAGIC_2, 0, &param);
    perror("monte");		/* monte reboot failed. */
    exit(1);
} 

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
